@@ -207,6 +207,52 @@ class CmdStanModelTest(CustomTestCase):
         self.assertIn('theta', model_info_include['parameters'])
         self.assertIn('included_files', model_info_include)
 
+    def test_compile_with_includes(self):
+        getmtime = os.path.getmtime
+        configs = [
+            ('add_one_model.stan', ['include-path']),
+            ('bernoulli_include.stan', []),
+        ]
+        for stan_file, include_paths in configs:
+            stan_file = os.path.join(DATAFILES_PATH, stan_file)
+            include_paths = [
+                os.path.join(DATAFILES_PATH, path) for path in include_paths
+            ]
+
+            # Compile for the first time.
+            model = CmdStanModel(
+                stan_file=stan_file,
+                compile=False,
+                stanc_options={"include-paths": include_paths},
+            )
+            with LogCapture(level=logging.INFO) as log:
+                model.compile()
+            log.check_present(
+                ('cmdstanpy', 'INFO', StringComparison('compiling stan file'))
+            )
+
+            # Compile for the second time, ensuring cache is used.
+            with LogCapture(level=logging.DEBUG) as log:
+                model.compile()
+            log.check_present(
+                ('cmdstanpy', 'DEBUG', StringComparison('found newer exe file'))
+            )
+
+            # Compile after modifying included file, ensuring cache is not used.
+            def _patched_getmtime(filename: str) -> float:
+                includes = ['divide_real_by_two.stan', 'add_one_function.stan']
+                if any(filename.endswith(include) for include in includes):
+                    return float('inf')
+                return getmtime(filename)
+
+            with LogCapture(level=logging.INFO) as log, patch(
+                'os.path.getmtime', side_effect=_patched_getmtime
+            ):
+                model.compile()
+            log.check_present(
+                ('cmdstanpy', 'INFO', StringComparison('compiling stan file'))
+            )
+
     def test_compile_force(self):
         if os.path.exists(BERN_EXE):
             os.remove(BERN_EXE)
@@ -299,13 +345,8 @@ class CmdStanModelTest(CustomTestCase):
 
     def test_model_syntax_error(self):
         stan = os.path.join(DATAFILES_PATH, 'bad_syntax.stan')
-        with LogCapture(level=logging.WARNING) as log:
-            logging.getLogger()
-            with self.assertRaises(ValueError):
-                CmdStanModel(stan_file=stan)
-        log.check_present(
-            ('cmdstanpy', 'WARNING', StringComparison(r'(?s).*Syntax error.*'))
-        )
+        with self.assertRaisesRegex(ValueError, r'.*Syntax error.*'):
+            CmdStanModel(stan_file=stan)
 
     def test_repr(self):
         model = CmdStanModel(stan_file=BERN_STAN)
